$TITLE ('GALEX/COS INTERNAL Routines')
$PAGELENGTH(59)
$SB
$DEBUG
;THIS IS INTERNAL.A51,                        LAST EDITED ::  7-5-100 10:57 pm 
;*****************************************************************************
;
;   III
;    I      N    N    TTTTT   EEEEEE   RRRRR    N    N     AA     L
;    I      NN   N      T     E        R    R   NN   N    A  A    L
;    I      N N  N      T     EEEEE    R    R   N N  N   A    A   L
;    I      N  N N      T     E        RRRRR    N  N N   AAAAAA   L
;    I      N   NN      T     E        R   R    N   NN   A    A   L
;   III     N    N      T     EEEEEE   R    R   N    N   A    A   LLLLLL
;
;*****************************************************************************
; dbb 11-18-98 changed DPU_DATA to TMR_DATA, because previous versions resulted
; in varying data-segment layouts.  While it works okay for the program executing
; its a crash when the older version resets, relying on internal memory having
; a specific layout.  Humph.
;
; dbb 12-1-98 updated timer isr to look at TMOD bit 0 to decide how many
;             interrupts per second, 21 vs ~168.  Code is initialized at
;             the faster rate, but you can slow the clock using TEST C.
;             The idea is to always set GPFLAG at one-second intervals.
;
; dbb 1-5-98 WARNING!! This file contains ISRs pointed-to by INITSEG
;            be very careful about relocating this file... for compatability
;            with FUSE isr table in v16200.
;
; dbb 12-02-99 Wrote, then disabled, code to increase the rate of GPFLAG setting,
;              which is normally 1Hz
;
; v16738 dbb 12-03-99 Support for high-speed hkp packets
; v16769 dbb 02-02-00 Removed debug code referencing TH1SET, which is being
;                     reused as the serial number variable for diagnostics.
; v16778 dbb 02-10-00 Filled unused space with jumps to trap
; v16803 dbb 03-07-00 Corrected PROJECT sense to COS=0, GALEX=1
; v16885 dbb 05-07-00 REORGANIZED! New memory and IVT layout, future
;                     uploads will not be compatable with older versions.
; v16902 dbb 05-17-00 Reassigned ESS_FLAG to control hkp packet size
; v17000 dbb 05-22-00 Code Review Version
;*****************************************************************************
$SET (INTERNAL)
$INCLUDE(common.inc)

         NAME INTERNAL

;*****************************************************************************
INTERNAL SEGMENT CODE PAGE
         RSEG INTERNAL
;
$EJECT
;*****************************************************************************
;
; Subroutine  IVTIMAGE
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; These indirect interupt vectors ensure that we can modify the ISR locations
; in memory to accomodate uploaded code updates.
; This code is copied to the IVT area at 8000h
    LJMP   TRAPWILD           ; 00H
IVTIMAGE:
    LJMP   RXD0ISR            ; 03H External 0
    LJMP   T0_ISR             ; 0BH Timer 0
    LJMP   RXD1ISR            ; 13H External 1
    LJMP   T1_ISR             ; 1BH Timer 1
    LJMP   SER_ISR            ; 23H Serial Port
    LJMP   T2_ISR             ; 2BH Timer 2
    LJMP   PCA_ISR            ; 33H PCA
    LJMP   AUX_ISR            ; 3BH Undefined (not available on 'FX)
    FROGS_CODE   20           ; nops fill extra space

TRAPWILD:
    REPORT_DIAG DIAG002C      ; Trap!
    JMP    REENTRY            ; this will never happen, right?
$EJECT
;*****************************************************************************
;
; Subroutine  RXD1READ
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
RXD1READ:                     ; fetch incoming from registers
    FETCHX RXD_HARDWARE+700h  ; point to MSB 24..31
    JMP    RXD_READ
RXD0READ:                     ; fetch incoming from registers
; fetch incoming from registers
    FETCHX RXD_HARDWARE+300h  ; point to MSB 24..31
RXD_READ:                     ; fetch incoming from registers
    MOV    CMDBUFF+3,A        ; bits 31..24
    DEC    DPH
    MOVX   A,@DPTR
    MOV    CMDBUFF+2,A        ; bits 23..16
    DEC    DPH
    MOVX   A,@DPTR
    MOV    CMDBUFF+1,A        ; bits 15..8
    DEC    DPH
    MOVX   A,@DPTR
    MOV    CMDBUFF+0,A        ; bits 7..0
    RET                       ; RXD_READ fetch incoming from registers

    FROGS_CODE   35           ; nops fill extra space
$EJECT
;*****************************************************************************
;
; Subroutine  RXD1ISR
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
RXD1ISR:                    ; generated by external 1 interrupt
    CLR    EX1                ; disable external 0 interrupt
    PUSH   ACC                ; save registers changed in this isr
    PUSHDPTR                  ; save it because register is used below
    CALL   RXD1READ           ; move data to internal memory CMDBUFF
    JMP    CHECKRXD

    FROGS_CODE   18           ; nops fill extra space

RXD0ISR:                    ; generated by external 0 interrupt
    CLR    EX0                ; disable external 0 interrupt
    PUSH   ACC                ; save registers changed in this isr
    PUSHDPTR                  ; save it because register is used below
    CALL   RXD0READ           ; move data to internal memory CMDBUFF

CHECKRXD:
; now we look for special functions in the 32-bits: 0x8000000 == reset
    MOV    A,CMDBUFF+3        ; get MSB of address part
    MOV    NOP_FLAG,C         ; save C flag because this is an interrupt
;    PUSH   PSW
    CJNE   A,#080h,N1TSPECIAL ; 80 in top byte indicates special function
    MOV    A,CMDBUFF+2        ; so we look to LSB of address
    JNZ    BAILOUT            ; and if nonzero, we simply ignore dword
    MOV    A,CMDBUFF+1        ; and we also require data parts to be zero
    JNZ    BAILOUT            ; else we ignore the dword
    MOV    A,CMDBUFF+0        ;  LSB data, too
    JNZ    BAILOUT            ;  must be zero to trigger special POR
    JMP    HARD_RESET         ; simulate a power-on reset
N1TSPECIAL:                   ; Valid dwords get further processing...

; check that address MSB is within expected bounds
    CJNE   A,#08h,NOTOBE      ; command addresses are 0040..083F,
    AJMP   MUSTBEOK           ; addresses 0800..08FF are okay
NOTOBE:
    JC     MUSTBEOK           ; addresses 0000..07FF are okay, too.

; Here we report any real errors in message traffic
    SETLEDS 060h              ; inform user of this error
    AJMP   BAILOUT            ; we don't store bad dwords
MUSTBEOK:

; then perform the implied function: copy data to external memory
    MOV    A,CMDBUFF+3        ; this is the destination address, high-part
; v16838 dbb 03-28-00 Now using 8k RAM for all cmd/hkp i/o
$IF PARTIAL
    ORL    A,#0E0h            ; map incoming byte to address E000..EFFF
$ELSE
    SPARE_CODE 2
$ENDIF
    MOV    DPH,A              ; use the masked address
    MOV    DPL,CMDBUFF+2      ; this is the destination address, low-part
    MOV    A,CMDBUFF+0        ; this is the data, low-part
    MOVX   @DPTR,A            ; low part stored in external memory
    INC    DPTR               ; point to high dest
    MOV    A,CMDBUFF+1             ; the high part of the data
    MOVX   @DPTR,A             ; high part stored in external memory

; clean up and exit
BAILOUT:
    MOV    C,NOP_FLAG         ; restore C flag because this is an interrupt
;    POP    PSW
    POPDPTR                    ; restore registers used above
    POP    ACC                    ; restore registers changed in this isr
    SETB   EX0                ; enable external 0 interrupt
    JB     PROJECT,NO3RD      ; check which project
    SETB   EX1                ; enable external 0 interrupt
NO3RD:
    SETB   RXD_FLAG           ; signal mainroutine to get rxdata
    RETI                      ; RXD_ISR

    FROGS_CODE   21           ; nops fill extra space
$EJECT
;*****************************************************************************
;
; Subroutine  T0_XFER
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;************************************
;*     Timer 0 Update Interrupt     *
;************************************
T0_XFER:
    INC    A                  ; A holds 0-based offset into T0_ISR
    MOVC   A,@A+PC            ; fetch the code below and return in A
    RET                       ; T0_XFER

;*****************************************************************************
T0_ISR:
    SETB   CLOCK_TICK         ; set on every interrupt
    DJNZ   GPTMR,T0_SKIP      ; timer decrements every 49.152 mS
    SETB   CLOCK_TOCK         ; set on every second
    CALL   ONE_SECOND         ; set timer to expire in one second
T0_SKIP:

    DJNZ   HKPTMR,TOCKSKIP    ; time to send another hkp packet?
    JNB    PROJECT,TOCKSKIP   ; we don't auto-update in COS/DCE
    SETB   GPFLAG             ; signal to check command & send hkp
    CALL   HKP_SECOND         ; set timer to expire in one second
TOCKSKIP:

    RETI                      ; T0_ISR

    FROGS_CODE    7           ; nops fill extra space
$EJECT
;*****************************************************************************
;
; Subroutine  E0_ISR
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;********** Just in case we ever do want to do interrupts... ********
E0_ISR:
    SPARE_CODE 7
    RETI

E1_ISR:
    SPARE_CODE 7
    RETI

T1_ISR:
    SPARE_CODE 7
    RETI

T2_ISR:
    SPARE_CODE 7
    RETI

PCA_ISR:
    SPARE_CODE 7
    RETI

AUX_ISR:
    SPARE_CODE 7
    RETI

SER_ISR:
    SPARE_CODE 7
    RETI                      ; SER_ISR

    FROGS_CODE    8           ; nops fill extra space
$EJECT
;*****************************************************************************
;
; Subroutine  INIT_T0
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
INIT_T0:            ; start interrupt-driven timer GPTMR
;********************************************************************
;*  Using TIMER 1 to Generate Baud Rates                            *
;*  Set Baudrate to 9600 Baud                                       *
;*                                                                  *
;*  Modified dbb 4/16/96 for 16MHz oscillator                       *
;*    16MHz/12 gives 1,333,333.333 timer-ticks per second           *
;*    9600 baud * 16 = 153,600 timer overflows per second, SMOD=1   *
;*    1.333 MHz / 0.1536 MHz = 8.68055555 timer-ticks per overflow  *
;*     so rounding up, we use -9 as the timer 1 reload value in TH1 *
;*                                                                  *
;*    Working backwards, baudrate = 16MHz / (12 * 16 * 9) = 9259.26 *
;*      or about 3.5% error (slow), which is supposedly tolerable.  *
;*                                                                  *
;* Note that TIMER 0 is set to free running 16-bit mode, so it will *
;* rollover every 49.152 mS and generate an interrupt, if enabled.  *
;* Note that TIMER 0 is set to free running 13-bit mode, so it will *
;* rollover every 6.144 mS and generate an interrupt, if enabled.  *
;*                                                                  *
;********************************************************************
; TMOD (Serial Port Control Register), high nibble: timer 1; low: timer 0
;                +---------- GATE: when set timer 1 runs
;                |                 only while INT1 is high
;                |+--------- C/T: 1=event, 0=interval
;                ||++======= M1,M0: 10=8-bit auto reload mode
;                ||||+------ GATE: when set timer 0 runs
;                |||||             only while INT1 is high
;                |||||+----- C/T: 1=event timer, 0=interval
;                ||||||++=== M1,M0: 13-bit timer mode
;                ||||||||
;                vvvvvvvv
    MOV    TMOD,#00100001B    ; C/T = 0, Mode = 2, 8-bit auto reload
; Note!  Turned M0 on to create 49 ticks/second behavior ala FUSE dbb 4-28-00

    MOV    PCON,#80H          ; Double baud rate, SMOD = 1
    MOV          TH1,#0F7H             ; -9 gives 9259 baud at 16MHz
    SETB   TR1                 ; start timer 1, TR1 is TCON.6
    SETB   TR0                 ; start timer 0, TR0 is TCON.4

; Setup timer 2 for CHECKMON, so TF2 will be set every 1ms
    MOV    RCAP2L,#LOW(RELOAD) ; low part of reload value, in 75us clock ticks
    MOV    RCAP2H,#HIGH(RELOAD) ; high part of reload value
    SETB   TR2                 ; start timer 2, TR2 is T2CON.2

; SCON (Serial Port Control Register)
;                 ++========= SM0,SM1: Mode 1, 8-bit UART
;                 ||+-------- SM2: applies to Modes 2 & 3
;                 |||+------- REN: receiver enabled
;                 ||||+------ TB8: 9th bit not used for parity?
;                 |||||+----- RB8: 9th bit not used for parity?
;                 ||||||+---- TI:  Tx interrupt flag, allow Tx
;                 |||||||+--- RI:  Rx interrupt flag
;                 ||||||||
;                 vvvvvvvv
    MOV          SCON,#01011010B    ; Init Serial Interface
;
    SETB   TR0                 ; start timer 0, TR0 is TCON.4
    SETB   ET0                ; enable timer 0 interrupt, IE.1
    CALL   HKP_SECOND             ; initialize HKPTMR
    CALL   ONE_SECOND             ; initialize GPTMR
    RET                       ; INIT_T0
$EJECT
;*****************************************************************************
;
; Subroutine  NEXTSECOND
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;********************************************************************
NEXTSECOND:
; set timer pointed-to by R0 to be zero one second from now
    PUSH   ACC
    MOV    A,TMOD             ; fetch current Timer Mode Control Register
    ANL    A,#1               ; look at bit 0, Timer 0 M0 Mode 0/1
    JZ     sFAST              ; if bit is 0, timer is running 8x fast
    MOV    @R0,#21            ; set the timer to expire in about 1 second
    AJMP   sSLOW
sFAST:
    MOV    @R0,#163           ; set the timer to expire in about 1 second
sSLOW:
    MOV    TH0,#0AEh          ; try to get very close to 1 second, total
    MOV    TL0,#0
    POP    ACC
    RET                       ; NEXTSECOND
$EJECT
;*****************************************************************************
;
; Subroutine  ONE_SECOND
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
ONE_SECOND:
; set GPTMR to be zero one second from now
    PUSH   0                  ; save R0 since this is called from interrupt
    MOV    R0,#GPTMR          ; pointer to R0
    CALL   NEXTSECOND
    POP    0                  ; save R0 since this is called from interrupt
    RET                       ; ONE_SECOND
$EJECT
;*****************************************************************************
;
; Subroutine  HKP_SECOND
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
HKP_SECOND:
; set HKPTMR to be zero one second from now
    PUSH   0                  ; save R0 since this is called from interrupt
    MOV    R0,#HKPTMR         ; pointer to R0
    CALL   NEXTSECOND
    POP    0                  ; save R0 since this is called from interrupt
    RET                       ; HKP_SECOND
$EJECT
;*****************************************************************************
;
; Subroutine  ; Ten seconds is represented by a number that depends on the timer mode
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
FEEDDOG MACRO
    PUSH   0                  ; we want to use these registers without
    PUSH   1                    ; considering where else they may be used.
; Ten seconds is represented by a number that depends on the timer mode:
;  At 6.144 ms/tick we require 1600 (640h) Timer 0 rollovers to get 10
;  seconds, while at 49.152 ms/tick we use 200 = 00c8 rollovers.
    MOV    A,TMOD             ; fetch current Timer Mode Control Register
    ANL    A,#1               ; look at bit 0, Timer 0 M0 Mode 0/1
    JZ     fFAST              ; if bit is 0, timer is running 8x fast
    MOV    R0,#0C8h
    MOV    R1,#0
    AJMP   fSLOW
fFAST:
    MOV    R0,#040h
    MOV    R1,#6
fSLOW:
; We add the appropriate number of ticks to the current value of the
; free-runing counter, the sum stored to a match register that is
; used in hardware to trigger a watchdog reset.
    CLR    CY                 ; reset carry flag for later add-overflow test
    MOV    A,CL               ; fetch current PCA timer value (lowbyte)
    ADDC   A,R0               ; update with possible carry
    MOV    CCAP4L,A           ; put result in CCAP4L
    MOV    A,CH               ; fetch current PCA timer value (high byte)
    ADDC   A,R1               ; update with possible carry
    MOV    CCAP4H,A           ; put result in CCAP4H
    POP    1
    POP    0
    ENDM                      ; FEEDDOG

;*****************************************************************************
FEED_DOG:                     ; schedule watchdog timeout reset event
    PUSH   ACC                ; lets try to be unobtrusive about this
    PUSH   PSW                ; lets try to be unobtrusive about this
    FEEDDOG
    POP    PSW                ; restore after use herein
    POP    ACC                ; restore after use herein
    RET                       ; FEED_DOG
$EJECT
;*****************************************************************************
;
; Subroutine  INIT_WD
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
INIT_WD:
    MOV    CMOD,#44h          ; Let PCA clock source be Timer 0 overflow
                              ;  and WDTE watchdog enabled!
    CALL   FEED_DOG           ;initial watchdog timeout reset event

    MOV    CCAPM4,#48h        ; CCAPM4 setup as WatchDog Timer
    SETB   CR                 ; turn on CR, enable PCA counter on
    SETB   WATCHDOG_STATE
    RET                       ; INIT_WD

INSTAT:
    MOV    C,RI               ; INPUT STATUS OF SERIAL INTERFACE
    RET                       ; INSTAT

OUTSTAT:
    MOV    C,TI               ; OUTPUT STATUS OF SERIAL INTERFACE
    RET                       ; OUTSTAT

INCHAR:
    MOV    A,SBUF             ; CHARACTER INPUT-ROUTINE
    RET                       ; INCHAR

OUTCHAR:
    MOV    SBUF,A             ; CHARACTER OUTPUT-ROUTINE
    RET                       ; OUTCHAR

CLR_TI:
    CLR    TI                 ; CLEAR SERIAL TRANSMIT INTERRUPT FLAG
    RET                       ; CLR_TI

SET_TI:
    SETB   TI                 ; SET SERIAL TRANSMIT INTERRUPT FLAG
    RET                       ; SET_TI

CLR_RI:
    CLR    RI                 ; CLEAR SERIAL RECEIVE INTERRUPT FLAG
    RET                       ; CLR_RI

CLR_SER_IE:
    CLR    ES                 ; CLEAR SERIAL INTERRUPT ENABLE FLAG
    RET                             ; CLR_SER_IE

SET_SER_IE:
    SETB   ES                 ; SET SERIAL INTERRUPT ENABLE FLAG
    RET                       ; SET_SER_IE
$EJECT
;*****************************************************************************
;
; Subroutine  HEXBIT
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
HEXBIT:
    ANL    A,#0Fh             ; mask out upper nibble
    INC    A                  ; offset past RET
    MOVC   A,@A+PC            ; fetch the character representation of nibble
    RET                       ; HEXBIT
    DB     '0123456789ABCDEF',0
$EJECT
;*****************************************************************************
;
; Subroutine  OUT_LEDS
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; bit pattern in A
OUT_LEDS:
    STOREX LEDS
    MOV    LEDimage,A         ; save the value output because we can't read back.
    RET                             ; OUT_LEDS

TOG_LEDS:
    XRL    LEDimage,A         ; save the value output because we can't read back.
    AJMP   UPD_LEDS

SET_LEDS:
    ORL    LEDimage,A         ; save the value output because we can't read back.
    AJMP   UPD_LEDS

CLR_LEDS:
    XRL    A,0FFh             ; toggle the bits
    ANL    LEDimage,A         ; save the value output because we can't read back.

UPD_LEDS:
    MOV    A,LEDimage         ; prepare to write back
    AJMP   OUT_LEDS
$EJECT
;*****************************************************************************
;
; Subroutine  IVTINDIR
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; Before turning on interrupts, we must load this data area table with code
IVTINDIR:                     ; copy interrupt vectors from code to 32k RAM
    MOV    R2,#LOW(IVTIMAGE)  ; source
    MOV    R3,#HIGH(IVTIMAGE) ; source
    MOV    R4,#LOW(vRXD0ISR)  ; dest
    MOV    R5,#HIGH(vRXD0ISR) ; dest
    MOV    R0,#1Ch            ; R0 holds number of bytes to move
    CLR    F0                 ; F0 selects CRC instead of actual move
    SETB   SPECIALF           ; source is Code
    CALL   BYTE_MOVE          ; copy command buffer to image
    CLR    SPECIALF           ; source is Data, by default
    RET                       ; IVTINDIR

    FROGS_CODE 4              ; nops for alignment
$EJECT
;*****************************************************************************
;
; Subroutine  DOCRC
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;***********************************
;*     do the CRC calculation      *
;***********************************
DOCRC:
    MOV    TEMP,A             ;save the received byte

    MOV    A,CRCL             ;swap high and low
    MOV    DXH,A
    MOV    A,CRCH

    XRL    A,TEMP             ;XOR with the data
    MOV    DXL,A

    SWAP   A
    ANL    A,#0FH             ;shift right 4

    XRL    A,DXL
    MOV    DXL,A

    SWAP   A
    ANL    A,#0F0H            ;shift left 4

    XRL    A,DXH
    MOV    DXH,A

    MOV    A,DXL
    SWAP   A
    MOV    TEMP1,A
    ANL    A,#0F0H
    CLR    C
    RLC    A
    XCH    A,TEMP1
    ANL    A,#0FH
    RLC    A                  ;shift left 5

    XRL    A,DXH
    MOV    CRCH,A
    MOV    A,TEMP1
    XRL    A,DXL
    MOV    CRCL,A

    MOV    A,TEMP             ;restore the data byte
    RET                       ; DOCRC

    FROGS_CODE   11           ; nops for alignment
$EJECT
;*****************************************************************************
;
; Subroutine  FILL_RAM_VALUE
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; Load a RAM range with a constant
FILL_RAM_VALUE:               ; value to fill passed in A
                              ; starting address is in DPTR
                              ; R2 contains the low-byte (ending address +1)
                              ; R3 contains the high-byte (ending address +1)
    PUSHDPTR
    PUSH   ACC                ; save test_value
FILL_LOOP:
    POP    ACC                ; restore test_value
    MOVX   @DPTR,A            ; load test_value into memory
    PUSH   ACC                ; save test_value
    INC    DPTR               ; point to the next address in RAM
    MOV    A,R3               ; prepare to compare the high-byte of DPTR
    CJNE   A,DPH,FILL_LOOP    ; look for lastaddr (high-byte)
    MOV    A,R2               ; prepare to compare the low-byte of DPTR
    CJNE   A,DPL,FILL_LOOP    ; look for lastaddr (low-byte)
    POP    ACC                ; clean up stack
    POPDPTR
    RET                       ; FILL_RAM_VALUE

    FROGS_CODE    5           ; nops for alignment
$EJECT
;*****************************************************************************
;
; Subroutine  ZAP_TS30
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
ZAP_TS30:
    MOV    R0,#40H            ; 64 bytes per subaddress
    MOV    DPTR,#TS30         ; Used for Diagnostics (informally, so far)
    CLR    A                  ; fill value
DO_AGIN9:
    MOVX   @DPTR,A            ; zap that location
    INC    DPTR               ; point to next location to zap
    DJNZ   R0,DO_AGIN9        ; and zap the entire subaddress
; Patch telemetry for new layout 4-3-97
    STOREX tDIAG0             ; 28 5 DIAG0
    STOREX tDIAG1           ; 28 5+ DIAG1
    STOREX tDIAG2           ; 28 9+ DIAG2
    RET                       ; ZAP_TS30
    FROGS_CODE    1           ; nops for alignment
$EJECT
;*****************************************************************************
;
; Subroutine  INIT_REGION
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
INIT_REGION:
    CALL   NEW_REGION         ; initialize MEMPTR and CRCx
    CLR    GF0                ; reset flag to detect BACKGROUND reentry
    RET                       ; INIT_REGION

;*****************************************************************************
; Since this routine is called as an initialization, we will initialize
;  CONTEXT here.
INIT_TEST_BKGRND:
    MOV    CONTEXT,#LOW(REGION5)
    MOV    CONTEXT+1,#HIGH(REGION5)
    JMP    INIT_REGION        ; INIT_TEST_BKGRND
$EJECT
;*****************************************************************************
;
; Subroutine  LOAD_MEMPTR
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
LOAD_MEMPTR:                  ; let DPTR point to a location to be processed
    MOV    DPL,MEMPTR
    MOV    DPH,MEMPTR+1
    RET                       ; LOAD_MEMPTR
$EJECT
;*****************************************************************************
;
; Subroutine  RM0
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; These are the methods for fetching bytes to be run through the CRC machine
RM0:                             ; read regular external memory
    CALL   LOAD_MEMPTR        ; let DPTR point to a location to be processed
    MOVX   A,@DPTR
    RET                       ; RM0

; Moved here to keep region data at the same address as in v162
RM1:                             ; read code space
    CALL   LOAD_MEMPTR        ; let DPTR point to a location to be processed
    CLR    A                  ; A <-- 0
    MOVC   A,@A+DPTR
    RET                       ; RM1
$EJECT
;*****************************************************************************
;
; Subroutine  LOAD_CONTEXT
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
LOAD_CONTEXT:                 ; Load DPTR with CONTEXT pointer
    MOV    DPL,CONTEXT
    MOV    DPH,CONTEXT+1
    RET                       ; LOAD_CONTEXT
$EJECT
;*****************************************************************************
;
; Subroutine  FETCHACC
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; Using the context pointer, we call the appropriate method above
FETCHACC:
    CALL   LOAD_CONTEXT       ; Load DPTR with CONTEXT pointer
    MOV    A,#5               ; offset of pointer high part
    MOVC   A,@A+DPTR
    PUSH   ACC
    MOV    A,#4               ; offset of pointer low part
    MOVC   A,@A+DPTR
    MOV    DPH,A
    POP    DPL
    CLR    A                  ; A <-- 0
    JMP    @A+DPTR            ; FETCHACC
$EJECT
;*****************************************************************************
;
; Subroutine  STORECRC
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
STORECRC:
    MOV     CRCSAVEL,CRCL
    MOV     CRCSAVEH,CRCH
    RET                       ; STORECRC
$EJECT
;*****************************************************************************
;
; Subroutine  NEW_REGION
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
NEW_REGION:                   ; initialize MEMPTR and CRCx
    PUSHDPTR                  ; save because we don't intend to change
; Init MEMPTR
    CALL   LOAD_CONTEXT       ; Load DPTR with CONTEXT pointer
    MOV    A,#1               ; offset of start index high part
    MOVC   A,@A+DPTR
    PUSH   ACC
    CLR    A                  ; offset of start index low part
    MOVC   A,@A+DPTR
    MOV    MEMPTR+1,A         ; save pointer to next location to process
    POP    MEMPTR             ; save pointer to next location to process
; Init CRC
    PRESETCRC
    CALL    STORECRC          ; stash current CRC value
    POPDPTR                   ; restore because we don't intend to change
    RET                       ; NEW_REGION
$EJECT
;*****************************************************************************
;
; Subroutine  LOADPTR
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; The CONTEXT structure includes some pointers which we would like to load
; into DPTR by specifying the byte offset into the structure, passed in A
    FROGS_CODE   13           ; nops for alignment

LOADPTR:
    CALL   LOAD_CONTEXT       ; Load DPTR with CONTEXT pointer
    PUSH   0
    MOV    R0,A                    ; stash oset here
    INC    A                  ; offset of image pointer high part
    MOVC   A,@A+DPTR
    PUSH   ACC
    MOV    A,R0               ; offset of image pointer low part
    MOVC   A,@A+DPTR
    MOV    DPH,A
    POP    DPL                ; DPTR points to Image
    POP    0
    RET                       ; LOADPTR
    FROGS_CODE    3           ; nops for alignment

LOAD_PTR MACRO oset
    MOV    A,#oset            ; offset of image pointer high part
    CALL   LOADPTR
    ENDM                      ; LOAD_PTR
$EJECT
;*****************************************************************************
;
; Subroutine  SAVE_CRC
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
SAVE_CRC:
    MOV    A,CRCL
    MOVX   @DPTR,A
    INC    DPTR
    MOV    A,CRCH
    MOVX   @DPTR,A
    RET                       ; SAVE_CRC
$EJECT
;*****************************************************************************
;
; Subroutine  SAVE_IMAGE
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
SAVE_IMAGE:
    LOAD_PTR 10                    ; DPTR points to Image
    CALL   SAVE_CRC
    RET                       ; SAVE_IMAGE
$EJECT
;*****************************************************************************
;
; Subroutine  BACKTRACK
;
; INPUTS -- none, GP0 flag may be set to disable CRC checking function
; OUTPUTS -- none, may modify some CRC registers
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
; Per Action Item 1.004: The CHECKRXDFLAG routine is called here to increase
;  the frequency of checking the receive command buffer to determine when to
;  copy the buffer to housekeeping and then start processing of the command.
;  The programmer chose to put the call here because BACKGROUND may be called
;  more often than once per mainloop, thereby improving command response
;  performance.  The Action Item implies that it may not be necessary to
;  call so frequently, but that we do so to satisfy an out-of-spec requirement
;  to handle commands at a rate greater than once per second.
;
;*****************************************************************************
BACKTRACK:
                              ; Check for a command opcode and set activity flag
    CALL   CHECKRXDFLAG       ; detect opcode, stash command buffer
    RET                       ; reentry not allowed, drop out.
BACKGROUND:
    CALL   BACKTRACK          ; be sure to check SIO and EX0 at least once
    JB     GF0,BACKTRACK      ; if reentry, then exit
    SETB   GF0                ; set flag to detect reentry
    PUSH   DPL
    PUSH   DPH
    MOV    CRCL,CRCSAVEL
    MOV    CRCH,CRCSAVEH
    CALL   LOAD_CONTEXT       ; Load DPTR with CONTEXT pointer
;
; if MEMPTR points to EndAddr, then the CRC calculation is complete
;  and so we output the CRC to the monitor, link to the next context and
;  initialize it by setting CRC to FFFF and MEMPTR to StartAddr.
; Otherwise we just calculate the CRC and increment MEMPTR.
    MOV    A,#3               ; offset of EndAddr, high byte
    MOVC   A,@A+DPTR          ; fetch high part of EndAddr
    CJNE   A,MEMPTR,sNOTDONE  ; maybe complete?
    MOV    A,#2               ; offset of EndAddr, low byte
    MOVC   A,@A+DPTR          ; fetch low part of EndAddr
    CJNE   A,MEMPTR+1,sNOTDONE ; complete?
    JMP    sALLDONE
sNOTDONE:                     ; short range label for NOTDONE
    JMP    NOTDONE

; CRC calc complete!  Stash result in monitor
sALLDONE:
    LOAD_PTR 6                ; DPTR points to TS
    CALL   SAVE_CRC
;
; Test if image is different from just calculated CRC.  If different,
; then update the image, and if Diag is non-zero, then report Diag.
    LOAD_PTR 10                    ; DPTR points to Image
    MOVX   A,@DPTR            ; fetch low byte of image CRC
    JNZ    notInit
    FETCHNEXT                     ; fetch high byte of image CRC
    JNZ    notInit
;     Image was zero, which is possible but unlikely.  If the CRC is zero,
;     then we won't report transition into or outof the zero CRC calculation.
;     We assume the zero simply means the CRC image was uninitialized, and
;     so a diagnostic message wouldn't be helpful.  So we simply save the
;     image and then go on to the next REGION
; Save Image
    CALL   SAVE_IMAGE
    JMP    NextLink

changed:
    CALL   SAVE_IMAGE

; If the diagnostic code is zero, then we suppress the report.
    CALL   LOAD_CONTEXT       ; Load DPTR with CONTEXT pointer
    MOV    A,#12              ; offset is 0
    MOVC   A,@A+DPTR          ; Diag Code is in the code space; load to DPTR
    JZ     NextLink           ; suppress warning if 0
;
    PUSH   ACC
    MOV    A,#13              ; offset is 0
    MOVC   A,@A+DPTR          ; fetch the 5A part, which marks it as a DIAG
    MOV    DPL,A              ;
    POP    DPH
    JZ     NextLink           ; suppress warning if 0

    LOAD_PTR 12                    ; DPTR points to Diagnostic
    CALL   REPORT_DIAGNOSTIC
NoDiag:
    JMP    NextLink

notInit:
    LOAD_PTR 10                    ; DPTR points to Image
    MOVX   A,@DPTR            ; fetch low byte of image CRC
    CJNE   A,CRCL,changed
    FETCHNEXT                     ; fetch high byte of image CRC
    CJNE   A,CRCH,changed

; Fetch link and assign as new context
NextLink:
    CALL   LOAD_CONTEXT       ; Load DPTR with CONTEXT pointer
    MOV    A,#9               ; offset of link pointer high part
    MOVC   A,@A+DPTR
    PUSH   ACC
    MOV    A,#8               ; offset of link pointer low part
    MOVC   A,@A+DPTR
    MOV    CONTEXT+1,A
    POP    CONTEXT

    CALL   NEW_REGION         ; initialize MEMPTR and CRCx

    JMP    ALLDONE
; Otherwise...
NOTDONE:
    CALL   FETCHACC
    CALL   DOCRC
    CALL   LOAD_MEMPTR        ; let DPTR point to a location to be processed
    INC    DPTR
    MOV    MEMPTR,DPL         ; save pointer to next location to process
    MOV    MEMPTR+1,DPH             ; save pointer to next location to process
ALLDONE:
    CALL    STORECRC          ; stash current CRC value
    POP     DPH
    POP     DPL
    CLR     GF0               ; reset flag to detect reentry
    JMP     BACKTRACK         ; call mon51 again
    RET                       ; BACKGROUND

    FROGS_CODE    5           ; nops for alignment
$EJECT
;*****************************************************************************
;
; Subroutine  FROMCODE
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
; External to external byte-wise move, up to 255 per call.
FROMCODE:
    CLR    A                  ; index always zero
    MOVC   A,@A+DPTR          ; get byte from src
    AJMP   BYTE_CONT          ; resume after fetching source byte

FROMC2DE:
    CLR    A                  ; index always zero
    MOVC   A,@A+DPTR          ; get byte from src
    AJMP   BYTE_C2NT          ; resume after fetching source byte

    FROGS_CODE    8           ; nops for alignment
$EJECT
;*****************************************************************************
;
; Subroutine  BYTE_MOVE
;
; INPUTS -- none
; OUTPUTS -- none
;
; v17044 dbb 07-01-00 Added a banner to every called subroutine       
;                       in response to Action Item 1.005                     *
;                                                                            *
;*****************************************************************************
BYTE_MOVE:                    ; R0 holds number of bytes to move
                              ; R3:R2 holds pointer to source
                              ; R5:R4 holds pointer to dest
                              ; F0 selects CRC instead of actual move
    JB     F0,CRCXONLY        ; select actual move, or just calculate CRC
BYTE_MOVX:
    MOV    DPL,R2             ; restore the source pointer
    MOV    DPH,R3             ; restore the source pointer

    JB     SPECIALF,FROMCODE
    MOVX   A,@DPTR            ; get byte from src
BYTE_CONT:
    INC    DPTR               ; point to next source byte
    MOV    R2,DPL             ; save source pointer
    MOV    R3,DPH             ; save source pointer
    MOV    DPL,R4             ; restore dest pointer
    MOV    DPH,R5             ; restore dest pointer
    JB     F0,CRCONLY         ; select actual move, or just calculate CRC

    MOVX   @DPTR,A            ; put byte in dest

    JMP    SKIPCRC            ; and don't calculate CRC
CRCONLY:
    CALL   DOCRC              ; sure, this takes time... but what the heck!

SKIPCRC:
    INC    DPTR               ; point to next dest byte
    MOV    R4,DPL             ; save the dest pointer
    MOV    R5,DPH             ; save the dest pointer
    DJNZ   R0,BYTE_MOVE       ; Keep grinding til all bytes are moved
                              ; On exit: A,R0,R2,R3,R4,and R5 are changed.
    JMP    END_MOVE

CRCXONLY:
    MOV    DPL,R2             ; restore the source pointer
    MOV    DPH,R3             ; restore the source pointer

CRCZONLY:
    JB     SPECIALF,FROMC2DE  ; get byte from code, if flagged.
    MOVX   A,@DPTR            ; get byte from src
BYTE_C2NT:
    CALL   DOCRC              ; sure, this takes time... but what the heck!

    INC    DPTR               ; point to next dest byte
    DJNZ   R0,CRCZONLY        ; Keep grinding til all bytes are moved
    MOV    R2,DPL             ; save source pointer
    MOV    R3,DPH             ; save source pointer

END_MOVE:
    NOP                       ; word alignment
    RET                       ; BYTE_MOVE

$IF DEGAP
    FROGS_CODE  876           ; nops fill extra space
    FROGS_CODE    5           ; nops fill extra space
    TRAP_CODE     1           ; INTERNAL SEGMENT CODE
$ENDIF

    END                       ; INTERNAL
